@@ -3,13 +3,15 @@
 # Licensed under the MIT License. See License.txt in the project root for
 # license information.
 # --------------------------------------------------------------------------
-# pylint: disable=too-few-public-methods, too-many-instance-attributes
-# pylint: disable=super-init-not-called, too-many-lines
+# pylint: disable=too-few-public-methods, too-many-instance-attributes, super-init-not-called, too-many-lines
 
+from urllib.parse import unquote
 from enum import Enum
 
+from azure.core import CaseInsensitiveEnumMeta
 from azure.core.paging import PageIterator
 from azure.core.exceptions import HttpResponseError
+
 from ._parser import _parse_datetime_from_str
 from ._shared.response_handlers import return_context_and_deserialized, process_storage_error
 from ._shared.models import DictMixin, get_enum_value
@@ -45,7 +47,7 @@ class Metrics(GeneratedMetrics):
     """
 
     def __init__(self, **kwargs):
-        self.version = kwargs.get('version', u'1.0')
+        self.version = kwargs.get('version', '1.0')
         self.enabled = kwargs.get('enabled', False)
         self.include_apis = kwargs.get('include_apis')
         self.retention_policy = kwargs.get('retention_policy') or RetentionPolicy()
@@ -218,9 +220,8 @@ class AccessPolicy(GenAccessPolicy):
     :param start:
         The time at which the shared access signature becomes valid. If
         omitted, start time for this call is assumed to be the time when the
-        storage service receives the request. Azure will always convert values
-        to UTC. If a date is passed in without timezone info, it is assumed to
-        be UTC.
+        storage service receives the request. The provided datetime will always
+        be interpreted as UTC.
     :type start: ~datetime.datetime or str
     """
     def __init__(self, permission=None, expiry=None, start=None):
@@ -334,10 +335,15 @@ class ShareProperties(DictMixin):
     :ivar int remaining_retention_days:
         To indicate how many remaining days the deleted share will be kept.
         This is a service returned value, and the value will be set when list shared including deleted ones.
+    :ivar int provisioned_bandwidth:
+        Provisioned bandwidth in megabits/second. Only applicable to premium file accounts.
     :ivar ~azure.storage.fileshare.models.ShareRootSquash or str root_squash:
         Possible values include: 'NoRootSquash', 'RootSquash', 'AllSquash'.
     :ivar list(str) protocols:
         Indicates the protocols enabled on the share. The protocol can be either SMB or NFS.
+    :ivar bool enable_snapshot_virtual_directory_access:
+        Specifies whether the snapshot virtual directory should be accessible at the root of the share
+        mount point when NFS is enabled. If not specified, it will be accessible.
     """
 
     def __init__(self, **kwargs):
@@ -356,10 +362,13 @@ class ShareProperties(DictMixin):
         self.provisioned_egress_mbps = kwargs.get('x-ms-share-provisioned-egress-mbps')
         self.provisioned_ingress_mbps = kwargs.get('x-ms-share-provisioned-ingress-mbps')
         self.provisioned_iops = kwargs.get('x-ms-share-provisioned-iops')
+        self.provisioned_bandwidth = kwargs.get('x-ms-share-provisioned-bandwidth-mibps')
         self.lease = LeaseProperties(**kwargs)
         self.protocols = [protocol.strip() for protocol in kwargs.get('x-ms-enabled-protocols', None).split(',')]\
             if kwargs.get('x-ms-enabled-protocols', None) else None
         self.root_squash = kwargs.get('x-ms-root-squash', None)
+        self.enable_snapshot_virtual_directory_access = \
+            kwargs.get('x-ms-enable-snapshot-virtual-directory-access')
     @classmethod
     def _from_generated(cls, generated):
         props = cls()
@@ -378,10 +387,12 @@ class ShareProperties(DictMixin):
         props.provisioned_egress_mbps = generated.properties.provisioned_egress_m_bps
         props.provisioned_ingress_mbps = generated.properties.provisioned_ingress_m_bps
         props.provisioned_iops = generated.properties.provisioned_iops
+        props.provisioned_bandwidth = generated.properties.provisioned_bandwidth_mi_bps
         props.lease = LeaseProperties._from_generated(generated)  # pylint: disable=protected-access
         props.protocols = [protocol.strip() for protocol in generated.properties.enabled_protocols.split(',')]\
             if generated.properties.enabled_protocols else None
         props.root_squash = generated.properties.root_squash
+        props.enable_snapshot_virtual_directory_access = generated.properties.enable_snapshot_virtual_directory_access
 
         return props
 
@@ -446,6 +457,7 @@ class Handle(DictMixin):
 
     All required parameters must be populated in order to send to Azure.
 
+    :keyword str client_name: Required. Name of the client machine where the share is being mounted.
     :keyword str handle_id: Required. XSMB service handle ID
     :keyword str path: Required. File or directory name including full path starting
      from share root
@@ -459,9 +471,12 @@ class Handle(DictMixin):
     :keyword ~datetime.datetime open_time: Required. Time when the session that previously opened
      the handle has last been reconnected. (UTC)
     :keyword ~datetime.datetime last_reconnect_time: Time handle was last connected to (UTC)
+    :keyword access_rights: Access rights of the handle.
+    :paramtype access_rights: Optional[List[Literal['Read', 'Write', 'Delete']]]
     """
 
     def __init__(self, **kwargs):
+        self.client_name = kwargs.get('client_name')
         self.id = kwargs.get('handle_id')
         self.path = kwargs.get('path')
         self.file_id = kwargs.get('file_id')
@@ -470,18 +485,21 @@ class Handle(DictMixin):
         self.client_ip = kwargs.get('client_ip')
         self.open_time = kwargs.get('open_time')
         self.last_reconnect_time = kwargs.get('last_reconnect_time')
+        self.access_rights = kwargs.get('access_right_list')
 
     @classmethod
     def _from_generated(cls, generated):
         handle = cls()
+        handle.client_name = generated.client_name
         handle.id = generated.handle_id
-        handle.path = generated.path
+        handle.path = unquote(generated.path.content) if generated.path.encoded else generated.path.content
         handle.file_id = generated.file_id
         handle.parent_id = generated.parent_id
         handle.session_id = generated.session_id
         handle.client_ip = generated.client_ip
         handle.open_time = generated.open_time
         handle.last_reconnect_time = generated.last_reconnect_time
+        handle.access_rights = generated.access_right_list
         return handle
 
 
@@ -584,7 +602,7 @@ class DirectoryProperties(DictMixin):
     @classmethod
     def _from_generated(cls, generated):
         props = cls()
-        props.name = generated.name
+        props.name = unquote(generated.name.content) if generated.name.encoded else generated.name.content
         props.file_id = generated.file_id
         props.file_attributes = generated.attributes
         props.last_modified = generated.properties.last_modified
@@ -612,7 +630,7 @@ class DirectoryPropertiesPaged(PageIterator):
     :ivar str location_mode: The location mode being used to list results. The available
         options include "primary" and "secondary".
     :ivar current_page: The current page of listed results.
-    :vartype current_page: list(dict(str, Any))
+    :vartype current_page: list[dict[str, Any]]
 
     :param callable command: Function to retrieve the next page of items.
     :param str prefix: Filters the results to return only directories whose names
@@ -722,7 +740,7 @@ class FileProperties(DictMixin):
     @classmethod
     def _from_generated(cls, generated):
         props = cls()
-        props.name = generated.name
+        props.name = unquote(generated.name.content) if generated.name.encoded else generated.name.content
         props.file_id = generated.file_id
         props.etag = generated.properties.etag
         props.file_attributes = generated.attributes
@@ -736,7 +754,7 @@ class FileProperties(DictMixin):
         return props
 
 
-class ShareProtocols(str, Enum):
+class ShareProtocols(str, Enum, metaclass=CaseInsensitiveEnumMeta):
     """Enabled protocols on the share"""
     SMB = "SMB"
     NFS = "NFS"
@@ -881,13 +899,17 @@ class ShareSasPermissions(object):
         an account SAS instead.
     :param bool list:
         List files and directories in the share.
+    :param bool create:
+        Create a new file in the share, or copy a file to a new file in the share.
     """
-    def __init__(self, read=False, write=False, delete=False, list=False):  # pylint: disable=redefined-builtin
+    def __init__(self, read=False, write=False, delete=False, list=False, create=False):  # pylint: disable=redefined-builtin
         self.read = read
+        self.create = create
         self.write = write
         self.delete = delete
         self.list = list
         self._str = (('r' if self.read else '') +
+                     ('c' if self.create else '') +
                      ('w' if self.write else '') +
                      ('d' if self.delete else '') +
                      ('l' if self.list else ''))
@@ -900,21 +922,22 @@ class ShareSasPermissions(object):
     def from_string(cls, permission):
         """Create a ShareSasPermissions from a string.
 
-        To specify read, write, delete, or list permissions you need only to
+        To specify read, create, write, delete, or list permissions you need only to
         include the first letter of the word in the string. E.g. For read and
         write permissions, you would provide a string "rw".
 
-        :param str permission: The string which dictates the read, write,
+        :param str permission: The string which dictates the read, create, write,
             delete, or list permissions
         :return: A ShareSasPermissions object
         :rtype: ~azure.storage.fileshare.ShareSasPermissions
         """
         p_read = 'r' in permission
+        p_create = 'c' in permission
         p_write = 'w' in permission
         p_delete = 'd' in permission
         p_list = 'l' in permission
 
-        parsed = cls(p_read, p_write, p_delete, p_list)
+        parsed = cls(p_read, p_write, p_delete, p_list, p_create)
 
         return parsed
 
@@ -1001,8 +1024,6 @@ class NTFSAttributes(object):
 
 
 def service_properties_deserialize(generated):
-    """Deserialize a ServiceProperties objects into a dict.
-    """
     return {
         'hour_metrics': Metrics._from_generated(generated.hour_metrics),  # pylint: disable=protected-access
         'minute_metrics': Metrics._from_generated(generated.minute_metrics),  # pylint: disable=protected-access
